#This software is a rerelease of PlotDataMakerGC.R module of the Genome Crawler
#Suite with alterations in the input and ouput format in order to improve
#transition between modules of eMESS.

#No part of this text or software may be used or reproduced in any matter
#whatsoever without written permission, except in the case of brief quotations
#embodied in critical articles of or for reviews.  For information address
#Brian W. Kirk (bwkbem@yahoo.com).
#Software Copyright (C) June 2003  Brian W. Kirk
#Form/Text Copyright (C) June 2003  Brian W. Kirk
#Released for use with R v1.5.1 (C)

#Copyright (C) June 2003  Brian W. Kirk
#Released for R v1.5.1 (C)

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

          
###############################################################################
#                                FUNCTIONS                                    #
###############################################################################


###############################################################################
#                           CONSTANT DECLARATION                              #
###############################################################################

#CONSTANTS AND VARIABLES OF TEMPORARY USE BEGIN WITH "x, y, or z" AND ALL
#USER DEFINED FUNCTIONS BEGIN WITH "f" or "g" IN ORDER TO FACILITATE REMOVABLE
#FROM MEMORY.  WHEN USING VARIABLES WITH THESE RESERVED INTIAL LETTERS,
#NEAREST CLEANUP SHOULD BE IDENTIFIED TO INSURE PROPER PERMANENCY


                 ############################################
                 #######  DEFINE PERMANENT CONSTANTS  #######
                 ############################################

#Define Input Directory and File
StatDataIn <- paste(StatDataDirectory, StatDataFile, sep="");

ClusterDataIn <- paste(CrawlerDirectory, CrawlerClusterFile, sep="");
GeneDataIn <- paste(CrawlerDirectory, CrawlerGeneFile, sep="");

GenomeDataIn <- paste(GenomeDirectory, GenomeFile, sep="");

#Define Output Dirctory with Prefix
OutputPrefix <- paste(OutputDirectory, OutputFile, sep="");


             #####################################################
             ##########  DEFINE TEMPORARY CONSTANTS  #############
             #####################################################

#Data Positions in CyberT output file
xStatRankPos <- 1;
xStatIDPos <- 2;
xStatWYpValuePos  <- 3;
xStatBonpValuePos <- 4;
xStattStatPos <- 5;
xStatLogFoldChngPos <- 6;
xStatStndErrPos <- 7;
xStatArrayNumberPos <- 8;
xStatIntensityPos <- 9;

#Data Positions in CrawlerClusterData output file
xClStartGeneNamePos <- 1; 
xClEndGeneNamePos <-  2;

xClClusterProbPos <- 3;

xClStartGenePosPos <- 4;
xClEndGenePosPos <- 5;
    
#Data Positions in CrawlerGeneData output file
GeGenePos <- 1;

GeExppValuePos <- 2;

GeFoldpValuePos <- 3;

GeClusterPosteriorPos <- 4;

GetStatPos <- 5;
GeLogFoldChngPos <- 6;
GeStndErrPos <- 7;
GeIntensityPos <- 8;
GeArrayNumberPos <- 9;

GeClusterPriorPos <- 10;
GeClusterLikelihoodPos <- 11;
GeClusterProbPos <- 12;

GeStartGeneNamePos <- 13; 
GeEndGeneNamePos <- 14;

GeStartGenePosPos <- 15;
GeEndGenePosPos <- 16;

###############################################################################
#                             VARIABLE DECLARATION                            #
###############################################################################

#CONSTANTS AND VARIABLES OF TEMPORARY USE BEGIN WITH "x", "y", or "z" AND ALL
#USER DEFINED FUNCTIONS BEGIN WITH "f" or "g" IN ORDER TO FACILITATE REMOVABLE
#FROM MEMORY.  WHEN USING VARIABLES WITH THESE RESERVED INTIAL LETTERS,
#NEAREST CLEANUP SHOULD BE IDENTIFIED TO INSURE PROPER PERMANCY


                        #####  Input Tables  ######

#Input StatSigData
xStatDataTable  <- data.frame(read.delim(StatDataIn,
                                       skip=StatHeaderLength));

#Input CrawlerClusterData
CrawlerClusterTable  <- data.frame(read.delim(ClusterDataIn,
                                       skip=CrawlerHeaderLength));

#Input CrawlerGeneData
CrawlerGeneTable  <- data.frame(read.delim(GeneDataIn,
                                       skip=CrawlerHeaderLength));

#Input Ordered Genes from Annotated Genome
xGenomeData <- data.frame(read.delim(GenomeDataIn,
                                       skip=GenomeHeaderLength));


                 #####  Create Vectors from Input Tables  #####

###### Data from StatSigTable

zStatRank <- as.numeric(as.vector(xStatDataTable[, xStatRankPos]));
zStatID <- as.character(as.vector(xStatDataTable[,xStatIDPos]));
zStatWYpValue  <- as.numeric(as.vector(xStatDataTable[, xStatWYpValuePos]));
zStatBonpValue <- as.numeric(as.vector(xStatDataTable[, xStatBonpValuePos]));
zStattStat <- as.numeric(as.vector(xStatDataTable[, xStattStatPos]));
zStatLogFoldChng <- as.numeric(as.vector(xStatDataTable[, xStatLogFoldChngPos]));
zStatStndErr <- as.numeric(as.vector(xStatDataTable[, xStatStndErrPos]));
zStatArrayNumber <- as.numeric(as.vector(xStatDataTable[, xStatArrayNumberPos]));
zStatIntensity <- as.numeric(as.vector(xStatDataTable[, xStatIntensityPos]));

#Find Stat Lambda
xLambdaScan <- scan(file=paste(StatDataDirectory, StatDataFile, sep=""),
                       what=character(0), sep="\t", nlines=StatHeaderLength);
xLambdaString <- grep("Lambda = ", xLambdaScan,
                          value=TRUE);

if (length(xLambdaString) != 0) {
  StatLambdaFound <- TRUE;
  xStringSplit <- unlist(strsplit(xLambdaString, "= "));
  StatLambda <- as.character(xStringSplit[2]); 
} else {
  StatLambdaFound <- FALSE;
  StatLambda <- "NotFound";
}

#Determine which pValue to Use from Cluster Gene Table Header
xBonferroniScan <- scan(file=GeneDataIn, what=character(0), sep="\t",
                        nlines=CrawlerHeaderLength);
xBonferroniString <- grep("Bonferroni = ", xBonferroniScan,
                          value=TRUE);

if (length(xBonferroniString) != 0) {
  StatBonferroniFound <- TRUE;
  xStringSplit <- unlist(strsplit(xBonferroniString, "= "));
  Bonferroni <- as.logical(xStringSplit[2]); 
} else {
  StatBonferroniFound <- FALSE;
  Bonferroni <- FALSE;
}

if (Bonferroni == TRUE) {
  zStatpValueIn <- zStatBonpValue 
} else {
  zStatpValueIn <- zStatWYpValue
}

#Extract Number of Permutations from Statistical Significance Table Header
if (!Bonferroni) {
  xPermNumberScan <- scan(file=paste(StatDataDirectory, StatDataFile, sep=""),
                       what=character(0), sep="\t", nlines=StatHeaderLength);
  xPermNumberString <- grep("Permutation Number = ", xPermNumberScan,
                          value=TRUE);

  if (length(xPermNumberString) != 0) {
    StatPermNumberFound <- TRUE;
    xStringSplit <- unlist(strsplit(xPermNumberString, "= "));
    StatPermNum <- as.numeric(xStringSplit[2]);
  
  } else {
    StatPermNumberFound <- FALSE;
    StatPermNum <- "Not Found";
  }
} else {
  StatPermNumberFound <- FALSE
}

#Set zero pValues from a permutation algorithm to the minimum value
xpValueSetUpFilter <- as.logical(match(zStatpValueIn, 0, nomatch=0));

if (StatPermNumberFound) {
   zStatpValueIn[xpValueSetUpFilter] <- 1/StatPermNum; 
} else {
  NonZeropValue <- zStatpValueIn[which(zStatpValueIn > 0)];
  minpValue <- min(NonZeropValue);
  
  zStatpValueIn[xpValueSetUpFilter] <- minpValue 
}


##### Data from CrawlerClusterTable

zClStartGeneName <- as.character(as.vector(CrawlerClusterTable[,
                                                      xClStartGeneNamePos])); 
zClEndGeneName <- as.character(as.vector(CrawlerClusterTable[,
                                                      xClEndGeneNamePos])); 
zClStartGenePos <- as.numeric(as.vector(CrawlerClusterTable[,
                                                      xClStartGenePosPos]));
zClEndGenePos <- as.numeric(as.vector(CrawlerClusterTable[,
                                                      xClEndGenePosPos]));   
zClClusterProb <- as.numeric(as.vector(CrawlerClusterTable[,
                                                         xClClusterProbPos]));

###### Data from CrawlerGeneTable

zGeGene <- as.character(as.vector(CrawlerGeneTable[, GeGenePos]));
zGeStartGeneName <- as.character(as.vector(CrawlerGeneTable[,
                                                      GeStartGeneNamePos]));
zGeEndGeneName <- as.character(as.vector(CrawlerGeneTable[,
                                                      GeEndGeneNamePos]));
zGeStartGenePos <- as.numeric(as.vector(CrawlerGeneTable[,
                                                      GeStartGenePosPos]));
zGeEndGenePos <- as.numeric(as.vector(CrawlerGeneTable[,
                                                         GeEndGenePosPos]));

zGeExppValue <- as.numeric(as.vector(CrawlerGeneTable[, GeExppValuePos]));

zGeFoldpValue <- as.numeric(as.vector(CrawlerGeneTable[,
                                                      GeFoldpValuePos]));

zGeClusterPosterior <- as.numeric(as.vector(CrawlerGeneTable[,
                                                    GeClusterPosteriorPos]));

zGeClusterPrior <- as.numeric(as.vector(CrawlerGeneTable[,
                                                       GeClusterPriorPos]));
zGeClusterLikelihood <- as.numeric(as.vector(CrawlerGeneTable[,
                                                   GeClusterLikelihoodPos]));
zGeClusterProb <- as.numeric(as.vector(CrawlerGeneTable[,
                                                         GeClusterProbPos]));
zGetStat <- as.numeric(as.vector(CrawlerGeneTable[, GetStatPos]));
zGeLogFoldChng <- as.numeric(as.vector(CrawlerGeneTable[,
                                                        GeLogFoldChngPos]));
zGeStndErr <- as.numeric(as.vector(CrawlerGeneTable[, GeStndErrPos]));
zGeIntensity <- as.numeric(as.vector(CrawlerGeneTable[, GeIntensityPos]));
zGeArrayNumber <- as.numeric(as.vector(CrawlerGeneTable[,
                                                         GeArrayNumberPos]));


##### Data from GenomeData
GenomeGenes <- as.character(as.vector(xGenomeData[,GenomeIDPos]));
GenomeIndex <- 1:length(GenomeGenes);


       ##### Create Vectors From StatSigData Relative To GenomeGenes #####

#Prime Vectors
GenomeRank <- rep("NA", length(GenomeGenes));

GenomepValueIn <- rep(1.0, length(GenomeGenes));
GenometStat <- rep(0, length(GenomeGenes));
GenomeLogFoldChng <- rep(0, length(GenomeGenes));

GenomeStndErr <- rep("NA", length(GenomeGenes));
GenomeIntensity <- rep("NA", length(GenomeGenes));
GenomeArrayNumber <- rep("NA", length(GenomeGenes));

#Substitute Values Corresponding to Genes Present in StatSigData
for (i in 1:length(zStatID)) {
  GeneFilter <- as.logical(match(GenomeGenes, zStatID[i], nomatch=0));
  
  GenomeRank[GeneFilter] <- zStatRank[i];
  GenomepValueIn[GeneFilter] <- zStatpValueIn[i];
  GenometStat[GeneFilter] <- zStattStat[i];

  GenomeLogFoldChng[GeneFilter] <- zStatLogFoldChng[i];
  GenomeStndErr[GeneFilter] <- zStatStndErr[i];
  GenomeIntensity[GeneFilter] <- zStatIntensity[i];
  GenomeArrayNumber[GeneFilter] <- zStatArrayNumber[i];
}

#Find QueryStart and QueryEnd
xQueryStartScan <- scan(file=GeneDataIn, what=character(0), sep="\t",
                        nlines=CrawlerHeaderLength);
xQueryStartString <- grep("Query Start = ", xQueryStartScan,
                          value=TRUE);

if (length(xQueryStartString) != 0) {
  QueryStartFound <- TRUE;
  xStringSplit <- unlist(strsplit(xQueryStartString, "= "));
  QueryStart <- as.character(xStringSplit[2]);
  QueryStartPos <- GenomeIndex[as.logical(match(GenomeGenes, QueryStart,
                                                 nomatch=0))];
} else {
  QueryStartFound <- FALSE;
  QueryStart <- as.character(GenomeGenes[1]);
  QueryStartPos <- 1;
}

xQueryEndScan <- scan(file=GeneDataIn, what=character(0), sep="\t",
                        nlines=CrawlerHeaderLength);
xQueryEndString <- grep("Query End = ", xQueryEndScan,
                          value=TRUE);

if (length(xQueryEndString) != 0) {
  QueryEndFound <- TRUE;
  xStringSplit <- unlist(strsplit(xQueryEndString, "= "));
  QueryEnd <- as.character(xStringSplit[2]);
  QueryEndPos <- GenomeIndex[as.logical(match(GenomeGenes, QueryEnd,
                                                 nomatch=0))]; 
} else {
  QueryEndFound <- FALSE;
  QueryEnd <- as.character(GenomeGenes[length(GenomeGenes)]);
  QueryEndPos <- length(GenomeGenes);
}




###############################################################################
#                               MAIN BODY                                     #
###############################################################################

# Identify Adjacent/Overlapping Regions of Statistically Significant Clusters
# and Amend "UberCluster" Information to CrawlerGeneTable
UberClusterStartGene <- "blank";
UberClusterStartPos <- 0;
UberClusterEndGene <- "blank";
UberClusterEndPos <- 0;

GeLength <- length(zGeGene);
ClLength <- length(zClClusterProb);
    
for (i in 1:GeLength) {
  GenePos <- GenomeIndex[as.logical(match(GenomeGenes, zGeGene[i],
                                               nomatch=0))];
  FrontPos <- zGeStartGenePos[i];
  BackPos <- zGeEndGenePos[i];

  #Use CrawlerClusterTable as Basis for Searching for UberClusters
  for (j in 1:ClLength) {
    if (GenePos >= zClStartGenePos[j] & GenePos <= zClEndGenePos[j]) {
      if (zClClusterProb[j] <= ClusterCut) {  
         if (zClStartGenePos[j] < FrontPos) {
            FrontPos <- zClStartGenePos[j];
         }
         if (zClEndGenePos[j] > BackPos) {
            BackPos <- zClEndGenePos[j]
         }
      }
    }
  }
  
  UberClusterStartPos <- c(UberClusterStartPos, FrontPos); 
  UberClusterStartGene <- c(UberClusterStartGene, GenomeGenes[FrontPos]);
  UberClusterEndPos <- c(UberClusterEndPos, BackPos);
  UberClusterEndGene <- c(UberClusterEndGene, GenomeGenes[BackPos]);
}
    
UberClusterStartGene <- UberClusterStartGene[2:length(UberClusterStartGene)];
UberClusterStartPos <- UberClusterStartPos[2:length(UberClusterStartPos)];
UberClusterEndGene <- UberClusterEndGene[2:length(UberClusterEndGene)];
UberClusterEndPos <- UberClusterEndPos[2:length(UberClusterEndPos)];

CrawlerGeneTable <- cbind(CrawlerGeneTable,
                       UberClusterStartGene, UberClusterEndGene,
                       UberClusterStartPos, UberClusterEndPos);


# Create a Table of PlotData Based on Individual Gene Information from
# CrawlerGeneTable

PlotData <- CrawlerGeneTable[1,]; #To be removed later only used to prime

GeGenef <- factor(zGeGene);
GeGenes <- levels(GeGenef);

for (i in GeGenes) {
  GeneFilter <- as.logical(match(zGeGene, i, nomatch=0));
  GeGeneData <- CrawlerGeneTable[GeneFilter,];
  GeGeneExppValue <- zGeExppValue[GeneFilter];
  
  if (min(GeGeneExppValue) < GeneCut) {
     pValueFilter <- order(GeGeneExppValue);
     OrderedGeGeneData <- GeGeneData[pValueFilter,];
     PlotData <- rbind(PlotData, OrderedGeGeneData[1,]);
  }                            
}

PlotData <- PlotData[(2:length(PlotData[,1])),];

rm(CrawlerGeneTable);

#Amend Plot Data with Genes Not in a Significant Cluster
zPDGene <- as.character(as.vector(PlotData[, GeGenePos]));
zPDExppValue <- as.numeric(as.vector(PlotData[, GeExppValuePos]));
zPDFoldpValue <- as.numeric(as.vector(PlotData[, GeFoldpValuePos]));
zPDClusterPosterior <- as.numeric(as.vector(PlotData[,
                                                     GeClusterPosteriorPos]));
zPDtStat <- as.numeric(as.vector(PlotData[, GetStatPos]));
zPDLogFoldChng <- as.numeric(as.vector(PlotData[, GeLogFoldChngPos]));
zPDStndErr <- as.numeric(as.vector(PlotData[, GeStndErrPos]));
zPDIntensity <- as.numeric(as.vector(PlotData[, GeIntensityPos]));
zPDArrayNumber <- as.numeric(as.vector(PlotData[, GeArrayNumberPos]));
zPDClusterPrior <- as.numeric(as.vector(PlotData[, GeClusterPriorPos]));
zPDClusterLikelihood <- as.numeric(as.vector(PlotData[,
                                                    GeClusterLikelihoodPos]));
zPDClusterProb <- as.numeric(as.vector(PlotData[, GeClusterProbPos]));
zPDStartGeneName <- as.character(as.vector(PlotData[, GeStartGeneNamePos]));
zPDEndGeneName <- as.character(as.vector(PlotData[, GeEndGeneNamePos]));
zPDStartGenePos <- as.numeric(as.vector(PlotData[, GeStartGenePosPos]));
zPDEndGenePos <- as.numeric(as.vector(PlotData[, GeEndGenePosPos]));
zPDUberClusterStartGene <- as.character(as.vector(PlotData[, 17]));
zPDUberClusterEndGene <- as.character(as.vector(PlotData[, 18]));
zPDUberClusterStartPos <- as.numeric(as.vector(PlotData[, 19]));
zPDUberClusterEndPos <- as.numeric(as.vector(PlotData[, 20]));    
   
for (i in QueryStartPos:QueryEndPos) {
  CandidateFilter <- as.logical(match(GenomeGenes[i], zPDGene, nomatch=0));
  if (sum(CandidateFilter) == 0) {
      # Amend PlotData Vectors
      zPDGene <- c(zPDGene, GenomeGenes[i]);
      zPDStartGeneName <- c(zPDStartGeneName, "NA");
      zPDEndGeneName <- c(zPDEndGeneName, "NA");
      zPDExppValue <- c(zPDExppValue, GenomepValueIn[i]);
      zPDFoldpValue <- c(zPDFoldpValue, GenomepValueIn[i]);
      zPDClusterPosterior <- c(zPDClusterPosterior, 1);
      zPDClusterPrior <- c(zPDClusterPrior, 1);
      zPDClusterLikelihood <- c(zPDClusterLikelihood, 1);
      zPDClusterProb <- c(zPDClusterProb, 1);
      zPDtStat <- c(zPDtStat, GenometStat[i]);
      zPDLogFoldChng <- c(zPDLogFoldChng, GenomeLogFoldChng[i]);
      zPDStndErr <- c(zPDStndErr, GenomeStndErr[i]);
      zPDIntensity <- c(zPDIntensity, GenomeIntensity[i]); 
      zPDArrayNumber <- c(zPDArrayNumber, GenomeArrayNumber[i]);
      zPDStartGenePos <- c(zPDStartGenePos, "NA");
      zPDEndGenePos <- c(zPDEndGenePos, "NA");
      zPDUberClusterStartGene <- c(zPDUberClusterStartGene, "NA");
      zPDUberClusterEndGene <- c(zPDUberClusterEndGene, "NA");
      zPDUberClusterStartPos <- c(zPDUberClusterStartPos, "NA");
      zPDUberClusterEndPos <- c(zPDUberClusterEndPos, "NA");      
  }
}

AmendPlotData <- cbind(zPDGene, zPDExppValue,
      zPDFoldpValue, zPDClusterPosterior, zPDtStat, zPDLogFoldChng,
      zPDStndErr, zPDIntensity, zPDArrayNumber, zPDClusterPrior,
      zPDClusterLikelihood, zPDClusterProb, zPDStartGeneName,
      zPDEndGeneName, zPDStartGenePos, zPDEndGenePos,
      zPDUberClusterStartGene, zPDUberClusterEndGene,
      zPDUberClusterStartPos, zPDUberClusterEndPos);    
        
# Order PlotData Relative to GenomeGenes
PlotDataGenes <- as.character(as.vector(AmendPlotData[,1]));
OrderedPlotData <- AmendPlotData[1,];
for (i in GenomeGenes[QueryStartPos:QueryEndPos]) {
  GenomeFilter <- as.logical(match(PlotDataGenes, i, nomatch=0));
  if (sum(GenomeFilter) > 0) {
    OrderedPlotData <- rbind(OrderedPlotData, AmendPlotData[GenomeFilter,])
  }
}

OrderedPlotData <- OrderedPlotData[2:length(OrderedPlotData[,1]),];



                       ############################
                       #####  OUTPUT RESULTS  #####
                       ############################


FilterHeader1 <- paste("StatData File = ", StatDataIn, sep="");
FilterHeader2 <- paste("Crawler Cluster File = ", ClusterDataIn, sep="");
FilterHeader3 <- paste("Crawler Gene File = ", GeneDataIn, sep="");
FilterHeader4 <- date();
FilterHeader5 <- c(paste("Cluster Cut = ", as.character(ClusterCut),
                                           sep=""));
FilterHeader6 <- c(paste("Gene Cut = ", as.character(GeneCut),
                                           sep=""));
FilterHeader7 <- c(paste("Query Start Found = ",
                         as.character(QueryStartFound), sep=""),
                   paste("Query Start = ",
                         as.character(QueryStart), sep=""),
                   paste("Query End Found = ",
                         as.character(QueryEndFound), sep=""),
                   paste("Query End = ", as.character(QueryEnd), sep=""));
FilterHeader8 <- c("Stat Param:",
                   paste("Bonferroni Found = ",
                         as.character(StatBonferroniFound), sep=""),
                   paste("Bonferroni = ", as.character(Bonferroni), sep=""),
                   paste("Stat Lamda Found = ",
                         as.character(StatLambdaFound), sep=""),
                   paste("Lamda = ",
                         as.character(StatLambda), sep=""));
FilterHeaderFill <- rep(" ", 1);
FilterHeaderTable <-  c("Gene", "Exp pValue", "Fold pValue", "Posterior",
                        "tStat", "LogFoldChng", "StndErr", "Intensity",
                        "ArrayNumber", "Prior", "Likelihood", "Clust Prob",
                        "ClusterStart", "ClusterEnd", "ClStartPos", "ClEndPos",
                        "UberStart", "UberEnd", "UberStartPos", "UberEndPos");

cat(FilterHeader1, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n");
cat(FilterHeader2, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n", append=TRUE);
cat(FilterHeader3, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n", append=TRUE);
cat(FilterHeader4, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n", append=TRUE);
cat(FilterHeader5, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n", append=TRUE);
cat(FilterHeader6, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n", append=TRUE);
cat(FilterHeader7, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n", append=TRUE);
cat(FilterHeader8, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n", append=TRUE);
cat(FilterHeaderFill, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n", append=TRUE);
cat(FilterHeaderTable, file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, "L", StatLambda,
                     ".BayPDM", sep=""), sep="\n", append=TRUE);

write.table(OrderedPlotData, file=paste(OutputPrefix, "L",
            StatLambda,".BayPDM",sep=""), quote=FALSE, sep="\t",
            col.names=FALSE, row.names=FALSE, append=TRUE);


rm(list=ls());
